#include <hmg.ch>

Function Main
local bColor
local nRandom := 0
local nGuess := 0
local cTime := time()
local cTitle := "HMG Sudoku" 
private aPuzzles := importfromtxt("sudoku.csv")
private aSudoku := {{0,0,0,2,0,3,8,0,1},;
                    {0,0,0,7,0,6,0,5,2},;
                    {2,0,0,0,0,0,0,7,9},;
                    {0,2,0,1,5,7,9,3,4},;
                    {0,0,3,0,0,0,1,0,0},;
                    {9,1,7,3,8,4,0,2,0},;
                    {1,8,0,0,0,0,0,0,6},;
                    {7,3,0,6,0,1,0,0,0},;
                    {6,0,5,8,0,9,0,0,0}}
private aOriginal := {}

if len(aPuzzles) > 0
   nRandom := val("0."+substr(cTime,8,1)+substr(cTime,5,1)+substr(cTime,8,2))
   nGuess := int(nRandom * len(aPuzzles))  
   if nGuess == 0
      nGuess := 1
   endif
   aSudoku := aclone(aPuzzles[nGuess])
   cTitle := "HMG Sudoku"+" Game no: "+alltrim(str(nGuess))+" of "+alltrim(str(len(aPuzzles)))
endif   

bColor := {||SudokuBackcolor()}

aOriginal := aclone(aSudoku)


define window Sudoku at 0,0 width 500 height 550 main title cTitle
   define grid square
      row 10
      col 10
      width 460
      height 445
      showheaders .f.
      headers {"","","","","","","","",""}
      widths {50,50,50,50,50,50,50,50,50}
      justify {2,2,2,2,2,2,2,2,2}
      allowedit .t.
      columncontrols {{"TEXTBOX","CHARACTER","9"},{"TEXTBOX","CHARACTER","9"},{"TEXTBOX","CHARACTER","9"},{"TEXTBOX","CHARACTER","9"},{"TEXTBOX","CHARACTER","9"},{"TEXTBOX","CHARACTER","9"},{"TEXTBOX","CHARACTER","9"},{"TEXTBOX","CHARACTER","9"},{"TEXTBOX","CHARACTER","9"}}
      fontsize 30
      dynamicbackcolor {bColor,bColor,bColor,bColor,bColor,bColor,bColor,bColor,bColor}
      columnwhen {{||entergrid()},{||entergrid()},{||entergrid()},{||entergrid()},{||entergrid()},{||entergrid()},{||entergrid()},{||entergrid()},{||entergrid()}}
      columnvalid {{||checkgrid()},{||checkgrid()},{||checkgrid()},{||checkgrid()},{||checkgrid()},{||checkgrid()},{||checkgrid()},{||checkgrid()},{||checkgrid()}}
      onchange checkpossiblevalues()     
      fontname "Arial"
      cellnavigation .t.
   end grid
   define label valid
      row 460
      col 10
      fontname "Arial"
      width 430
      fontsize 18
   end label
   define button next
      row 460
      col 440
      width 40
      caption "Next"
      action nextgame()
   end button    
end window
on key ESCAPE of Sudoku action Sudoku.release()
refreshsudokugrid()
sudoku.center
sudoku.activate
Return nil

function SudokuBackColor
do case
   case this.cellrowindex <= 3 // first row
      do case
         case this.cellcolindex <= 3 // first col
            return {200,100,100}
         case this.cellcolindex > 3 .and. this.cellcolindex <= 6// second col
            return {100,200,100}
         case this.cellcolindex > 6 // third col
            return {100,100,200}
      endcase
   case this.cellrowindex > 3 .and. this.cellrowindex <= 6  // second row       
      do case
         case this.cellcolindex <= 3 // first col
            return {100,200,100}
         case this.cellcolindex > 3 .and. this.cellcolindex <= 6// second col
            return {200,200,100}
         case this.cellcolindex > 6 // third col
            return {100,200,100}
      endcase
   case this.cellrowindex > 6 // third row       
      do case
         case this.cellcolindex <= 3 // first col
            return {100,100,200}
         case this.cellcolindex > 3 .and. this.cellcolindex <= 6// second col
            return {100,200,100}
         case this.cellcolindex > 6 // third col
            return {200,100,100}
      endcase
endcase
return nil           

function refreshsudokugrid
local i := 0
local aLine := {}
local aValue := sudoku.square.value
sudoku.square.deleteallitems()
if len(aSudoku) == 9
   for i := 1 to len(aSudoku)
      asize(aLine,0)
      for j := 1 to len(aSudoku[i])
         if aSudoku[i,j] > 0
            aadd(aLine,str(aSudoku[i,j],1,0))
         else
            aadd(aLine,'')
         endif
      next j
      sudoku.square.additem(aLine)   
   next i
endif
sudoku.square.value := aValue
return nil

function entergrid
local aValue := sudoku.square.value
if len(aValue) > 0
  if aOriginal[aValue[1],aValue[2]] > 0
      return .f.
  else
      return .t.
  endif
endif
return .f.             

function checkgrid
local nRow := this.cellrowindex
local nCol := this.cellcolindex
local nValue := val(alltrim(this.cellvalue))
local aLine := {}
local i := 0
local j := 0
local nRowStart := (int((nRow-1)/3) * 3) + 1
local nRowEnd := nRowstart + 2
local nColStart := (int((nCol-1)/3) * 3) + 1
local nColEnd := nColstart + 2
if nValue == 0
   this.cellvalue := ''
   sudoku.valid.value := ''
   aSudoku[nRow,nCol] := 0
   return .t.
endif
if nValue == aSudoku[nRow,nCol]
   return .t.
endif 
for i := 1 to 9
   if aSudoku[nRow,i] == nValue //row checking
      return .f.
   endif
   if aSudoku[i,nCol] == nValue //col checking
      return .f.
   endif
next i
for i := nRowStart to nRowEnd
   for j := nColStart to nColEnd
      if aSudoku[i,j] == nValue
         return .f.
      endif
   next j
next i
sudoku.valid.value := ''
aSudoku[nRow,nCol] := nValue
checkcompletion()
return .t.
   

function checkcompletion
for i := 1 to len(aSudoku)
   for j := 1 to len(aSudoku[i])
      if aSudoku[i,j] == 0
         return nil
      endif
   next j
next i
msginfo("Congrats! You won!")
return nil

function checkpossiblevalues
local aValue := sudoku.square.value
local aAllowed := {}
local cAllowed := ""
local nRowStart := (int((aValue[1]-1)/3) * 3) + 1
local nRowEnd := nRowstart + 2
local nColStart := (int((aValue[2]-1)/3) * 3) + 1
local nColEnd := nColstart + 2
local lAllowed := .t.
local i := 0
local j := 0
local k := 0
if aValue[1] > 0 .and. aValue[2] > 0
   if aOriginal[aValue[1],aValue[2]] > 0
      sudoku.valid.value := ""
      return nil
   else
      for i := 1 to 9
         lAllowed := .t.
         for j := 1 to 9
            if aSudoku[aValue[1],j] == i
               lAllowed := .f.
            endif
            if aSudoku[j,aValue[2]] == i
               lAllowed := .f.
            endif
         next j
         for j := nRowStart to nRowEnd
            for k := nColStart to nColEnd
               if aSudoku[j,k] == i
                  lAllowed := .f.
               endif
            next k
         next j
         if lAllowed
            aadd(aAllowed,i)
         endif
      next i
      if len(aAllowed) > 0
         for i := 1 to len(aAllowed)
            if i == 1
               cAllowed := cAllowed + alltrim(str(aAllowed[i]))
            else
               cAllowed := cAllowed + ", "+ alltrim(str(aAllowed[i]))
            endif
         next i
         sudoku.valid.value := "Possible Numbers: "+cAllowed
      else
         sudoku.valid.value := "Possible Numbers: Nil"
      endif               
      return nil
   endif
endif
return nil

function importfromtxt(cFilename)
local aLines := {}
local handle := fopen(cFilename,0)
local size1 := 0
local size2 := 0
local sample := 0
local lineno := 0
local eof1 := .f.
local linestr := ""
local c := ""
local len1 := 0
local x := 0
local finished := .f.
local m := 0
local length1 := 0
local linecount := 0
local length2 := 0
local aPuzzles := {}
local aPuzzle := {}
local aRow := {}
if handle == -1
   return aPuzzles
endif
size1 := fseek(handle,0,2)
size2 := size1
if size1 > 65000
   sample := 65000
else
   sample := size1
endif
fseek(handle,0)
lineno := 1
aadd(aLines,"")
c := space(sample)
eof1 := .f.
linestr := ""
len1 := 0
do while .not. eof1
   x := 0
   x := fread(handle,@c,sample)
   len1 := len1 + sample
   if x < 1
      eof1 := .t.
      aLines[lineno] := linestr
   else
      finished := .f.
      do while .not. finished
         m := at(chr(13),c)
         if m > 0
            if m == 1
               linestr := ""
               lineno := lineno + 1
               aadd(aLines,"")
               if asc(substr(c,m+1,1)) == 10
                  c := substr(c,m+2,len(c))
               else
                  c := substr(c,m+1,len(c))
               endif   
            else
               if len(alltrim(linestr)) > 0
                  linestr := linestr + substr(c,1,m-1)
               else
                  linestr := substr(c,1,m-1)
               endif
               if asc(substr(c,m+1,1)) == 10
                  c := substr(c,m+2,len(c))
               else
                  c := substr(c,m+1,len(c))
               endif   
               aLines[lineno] := linestr
               linestr := ""
               lineno := lineno + 1
               aadd(aLines,"")
            endif
         else
            linestr := c
            finished := .t.
         endif
      enddo
      c := space(sample)
   endif
enddo
fclose(handle)
for i := 1 to len(aLines)
   x := 1
   asize(aPuzzle,0)
   for j := 1 to 9 
      asize(aRow,0)
      for k := 1 to 9
         aadd(aRow,val(alltrim(substr(aLines[i],x,1))))
         x := x + 2
      next k
      if len(aRow) == 9
         aadd(aPuzzle,aclone(aRow))
      endif   
   next j
   if len(aPuzzle) == 9
      aadd(aPuzzles,aclone(aPuzzle))
   endif   
next i
return aPuzzles                   

function nextgame
local cTime := time()
local nRandom := 0
local nGuess := 0
if len(aPuzzles) > 0
   nRandom := val("0."+substr(cTime,8,1)+substr(cTime,5,1)+substr(cTime,8,2))
   nGuess := int(nRandom * len(aPuzzles))  
   if nGuess == 0
      nGuess := 1
   endif
   aSudoku := aclone(aPuzzles[nGuess])
   aOriginal := aclone(aSudoku)
   sudoku.title := "HMG Sudoku"+" Game no: "+alltrim(str(nGuess))+" of "+alltrim(str(len(aPuzzles))) 
endif   
refreshsudokugrid()
return nil